/*
 * Copyright 2016 - 2024 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include <ox/claw/claw.hpp>
#include <ox/fs/fs.hpp>

#include "validation.hpp"

namespace keel {

constexpr auto K1HdrSz = 40;

ox::Result<ox::UUID> readUuidHeader(ox::BufferView buff) noexcept;

ox::Error writeUuidHeader(ox::Writer_c auto &writer, ox::UUID const&uuid) noexcept {
	OX_RETURN_ERROR(write(writer, "K1;"));
	OX_RETURN_ERROR(uuid.toString(writer));
	return writer.put(';');
}

template<typename T>
ox::Result<T> readAsset(ox::BufferView buff) noexcept {
	std::size_t offset = 0;
	const auto err = readUuidHeader(buff).error;
	if (!err) {
		offset = K1HdrSz; // the size of K1 headers
	}
	auto out = ox::readClaw<T>(buff + offset);
	OX_RETURN_ERROR(out);
	OX_RETURN_ERROR(ensureValid(out.value));
	return out;
}

ox::Result<ox::ModelObject> readAsset(ox::TypeStore &ts, ox::BufferView buff) noexcept;

struct AssetHdr {
	ox::UUID uuid;
	ox::ClawHeader clawHdr;
};

ox::Result<ox::StringView> readAssetTypeId(ox::BufferView buff) noexcept;

ox::Result<AssetHdr> readAssetHeader(ox::BufferView buff) noexcept;

}
